@@ -11,10 +11,11 @@
 #' @param max.controls Like \code{min.controls} sets maximum group sized displayed with respect to the number of controls. Raise this value to see more groups.
 #' @param quantiles A points in the ECDF at which the distances between units will be displayed.
 #' @return \code{optmatch.summary}
+#' @seealso \code{\link{print.optmatch}}
 #' @method summary optmatch
 #' @S3method summary optmatch
 #' @rdname optmatch
-summary.optmatch <- function(object, 
+summary.optmatch <- function(object,
                              propensity.model = NULL, ...,
                              min.controls=.2, max.controls=5,
                              quantiles=c(0,.5, .95, 1)
@@ -26,26 +27,36 @@ summary.optmatch <- function(object,
 ## overall balance -- xBalance()
   so <- list()
   so$thematch <- object
-  so$matching.failed <- mfd <- is.na(object)
+  mfd <- is.na(object)
   if (all(mfd))
     {
       class(so) <- "summary.optmatch"
+      so$matching.failed <- table(attr(object, "subproblem"), attr(object, "contrast.group"))
+      dimnames(so$matching.failed)[[2]] <- c("z==0", "z==1")
       so$warnings <- c(so$warnings,
                        list("Matching failed.  (Restrictions impossible to meet?)\nEnter ?matchfailed for more info.")
                        )
       return(so)
     }
-  so$matched.set.structures <- stratumStructure(object[!mfd, drop=TRUE],min.controls=min.controls,max.controls=max.controls)
+  subprobs <- attr(object, "subproblem")
+  match.succeed <- tapply(mfd, subprobs, function(x) !all(x))
+  so$matching.failed <- table(attr(object, "subproblem"), attr(object, "contrast.group"), exclude=names(match.succeed)[match.succeed])
+  if (prod(dim(so$matching.failed)) == 0) {
+    so$matching.failed <- NULL
+  } else {
+    dimnames(so$matching.failed)[[2]] <- c("z==0", "z==1")
+  }
+  so$matched.set.structures <- stratumStructure(object,min.controls=min.controls,max.controls=max.controls)
   so$effective.sample.size <- attr(so$matched.set.structures, "comparable.num.matched.pairs")
 
   matchdists <- attr(object, "matched.distances")[levels(object[!mfd, drop=TRUE])]
   matchdists <- unlist(matchdists)
   so$total.distance <- sum(matchdists)
-  so$total.tolerances <- sum(attr(object, "exceedances"))
+  so$total.tolerances <- sum(unlist(attr(object, "exceedances")))
   so$matched.dist.quantiles <- quantile(matchdists, prob=quantiles)
 
   ## optional call to xbalance if it is loaded
-  if(exists("xBalance") && 
+  if(exists("xBalance") &&
      !is.null(propensity.model) &&
      inherits(propensity.model, "glm")) {
 
@@ -53,7 +64,7 @@ summary.optmatch <- function(object,
     # we warn, instead of an error, but the user may get an error
     # from model.frame or later
     if(is.null(propensity.model$model)) {
-      warning("This propensity seems to have been fit with 'model=FALSE'.\nI'm reconstructing the data set as best I can, but I might fail,\nor get a different data set than the one behind propensity.model.\nTo be sure, re-fit your propensity.model with 'model = TRUE'.")  
+      warning("This propensity seems to have been fit with 'model=FALSE'.\nI'm reconstructing the data set as best I can, but I might fail,\nor get a different data set than the one behind propensity.model.\nTo be sure, re-fit your propensity.model with 'model = TRUE'.")
     }
 
     # we need to handle the different ways of creating glm objects
@@ -75,21 +86,21 @@ summary.optmatch <- function(object,
     }
 
     if (is.null(modelData)) {
-      stop("summary.optmatch does not know how to process this type of model. Please file a bug report at https://github.com/markmfredrickson/optmatch/issues showing how you created your glm model.")  
+      stop("summary.optmatch does not know how to process this type of model. Please file a bug report at https://github.com/markmfredrickson/optmatch/issues showing how you created your glm model.")
     }
-    
+
     strata <- object[!mfd, drop=TRUE]
     data <- modelData[!mfd,]
 
     if (length(strata) != dim(data)[1]) {
-      stop("'summary' method unable to recreate data. Consider passing 'data' argument to 'pairmatch' or 'fullmatch'.")  
+      stop("'summary' method unable to recreate data. Consider passing 'data' argument to 'pairmatch' or 'fullmatch'.")
     }
 
     so$balance <- RItools::xBalance(fmla = formula(propensity.model),
                            strata = strata,
                            data = data,
                            report = c('adj.means', 'z.scores', 'chisquare.test'),
-                           na.rm = na.behavior) 
+                           na.rm = na.behavior)
 
   } else if (!is.null(propensity.model)) so$warnings <-
     c(so$warnings,
@@ -103,17 +114,23 @@ summary.optmatch <- function(object,
 print.summary.optmatch <- function(x,  digits= max(3, getOption("digits")-4),...)
   {
   if ('warnings' %in% names(x)) warns <- c(x$warnings, sep="\n")
-  if (all(x$matching.failed))
+
+  numsubprob <- length(levels(attr(x$thematch, "subproblem")))
+  numsubprobfail <- if (is.null(x$matching.failed)) 0 else nrow(x$matching.failed)
+  numobs <- length(x$thematch)
+  numobsfail <- sum(x$matching.failed)
+
+  if (numsubprob == numsubprobfail)
     {
       do.call(cat, warns)
       return(invisible(x))
     }
-  
-    if (any(x$matching.failed))  {
-      cat(paste("Matching failed in subclasses containing",sum(x$matching.failed),
-                "of",length(x$matching.failed),"observations.\n"))
-      cat("Reporting on subclasses where matching worked. (Enter ?matchfailed for more info.)\n")
-    } 
+
+  if (numsubprobfail > 0)  {
+    cat(paste("Matching succeeded in", numsubprob - numsubprobfail,"of", numsubprob,
+              "subgroups, accounting for", numobs - numobsfail, "of", numobs, "total observations.\n"))
+    cat("(Enter ?matchfailed for more info.)\n\n")
+  }
 
   attr(x$matched.set.structures, "comparable.num.matched.pairs") <- NULL
   cat("Structure of matched sets:\n")
