@@ -5,10 +5,10 @@
 ### toZ: turn various objects into a logical vector indicating treatment
 ### accepts a variety of inputs and keeps names/rownames if present
 
-setGeneric("toZ", function(x) {   
-  
+setGeneric("toZ", function(x) {
+
   if (any(is.na(x))) {
-    stop("NAs not allowed in treatment indicator.")  
+    stop("NAs not allowed in treatment indicator.")
   }
 
   if (is.data.frame(x) | is.matrix(x)) {
@@ -24,9 +24,9 @@ setGeneric("toZ", function(x) {
 
   if (length(unique(x)) != 2) {
     stop(paste("Treatment indicator must have exactly 2 levels not",
-      length(unique(x))))  
+      length(unique(x))))
   }
-  
+
   nms <- names(x)
   tmp <- standardGeneric("toZ")
   names(tmp) <- nms
@@ -42,8 +42,25 @@ setMethod("toZ", "numeric", function(x) as.logical(x))
 setMethod("toZ", "character", function(x) toZ(as.factor(x)))
 
 setMethod("toZ", "factor", function(x) {
-  toZ(as.numeric(x) - 1)  
+  toZ(as.numeric(x) - 1)
 })
 
-
-
+#' (Internal) Remove the call before digesting a distance so things like
+#' omitting caliper and calling caliper=NULL give the same digest
+#'
+#' @param dist Distance object to hash. Must be one of
+#' \code{InfinitySparseMatrix}, \code{BlockedInfinitySparseMatrix},
+#' \code{DenseMatrix}, \code{matrix}, or \code{distmatch.dlist}.
+#' @return Hash on the distance object with a null \code{call}
+#' @author Josh Errickson
+#' @import digest
+dist_digest <- function(dist) {
+  if (class(dist)[1] %in% c("InfinitySparseMatrix", "BlockedInfinitySparseMatrix", "optmatch.dlist", "DenseMatrix", "matrix")) {
+    csave <- attr(dist, "call")
+    attr(dist, "call") <- NULL
+    out <- digest(dist)
+    attr(dist, "call") <- csave
+    return(out)
+  }
+  stop("Must pass distance object")
+}
