@@ -1,6 +1,6 @@
 #' @include feasible.R
 
-.onAttach <- function(lib, pkg) {   
+.onAttach <- function(lib, pkg) {
 packageStartupMessage(paste(
 "You're loading optmatch, by B. Hansen and M. Fredrickson.\n",
 "The optmatch package makes essential use of D. P. Bertsekas\n",
@@ -11,10 +11,9 @@ packageStartupMessage(paste(
 "Dr. Bertsekas. For more information, enter\n",
 "relaxinfo() at the command line.\n"
 ))
-} 
+}
 
 .onLoad <- function(lib, pkg) {
   setFeasibilityConstants()
+  setTryRecovery()
 }
-
-
