@@ -18,11 +18,11 @@ test_that("No cross strata matches", {
 })
 
 test_that("Basic Matches", {
-  position <- rep(1:4, each = 4)  
+  position <- rep(1:4, each = 4)
   z <- rep(0:1, 8)
   names(z) <- letters[1:16]
   dist <- match_on(z ~ position, inv.scale.matrix = diag(1))
-  
+
   res.mat <- fullmatch(dist)
   res.ism <- fullmatch(as.InfinitySparseMatrix(dist))
   expect_equivalent(res.mat, res.ism)
@@ -56,13 +56,13 @@ test_that("Checks input", {
 
   # add only colnames
   m <- matrix(1:8, nrow = 2, ncol = 4)
-  colnames(m) <- LETTERS[3:6]  
+  colnames(m) <- LETTERS[3:6]
   expect_error(fullmatch(m))
   expect_error(fullmatch(as.InfinitySparseMatrix(m)))
 
   # repeat for rownames
   m <- matrix(1:8, nrow = 2, ncol = 4)
-  rownames(m) <- LETTERS[1:2]  
+  rownames(m) <- LETTERS[1:2]
   expect_error(fullmatch(m))
   expect_error(fullmatch(as.InfinitySparseMatrix(m)))
 
@@ -82,8 +82,8 @@ test_that("Checks input", {
   # which might be more than 1 if using exactMatch, e.g.
 
   m <- matrix(1, nrow = 2, ncol = 2, dimnames = list(c("a", "b"), c('d',
-  'e')))  
-  
+  'e')))
+
   expect_error(fullmatch(m, min.controls = c(0,0)))
   expect_error(fullmatch(m, max.controls = c(Inf,Inf)))
   expect_error(fullmatch(m, omit.fraction = c(1, 1)))
@@ -104,7 +104,7 @@ test_that("Reversion Test: Fullmatch handles omit.fraction for matrices", {
   A <- matrix(c(1,1,Inf,1,1,Inf,1,1,Inf,1,1,Inf), nrow = 3)
   dimnames(A) <- list(1:3, 4:7)
   Ai <- as.InfinitySparseMatrix(A)
-  
+
   # the omit.fraction values as computed by pairmatch and was the same
   # for both A and Ai
 
@@ -112,14 +112,14 @@ test_that("Reversion Test: Fullmatch handles omit.fraction for matrices", {
   res.ai <- fullmatch(Ai, min.controls = 1, max.controls = 1, omit.fraction = 0.5)
 
   expect_equivalent(res.a, res.ai)
-  
+
 })
 
 test_that("Reversion Test: Inf entries in matrix", {
   # this was handled in the previous version just fine
   d <- matrix(c(1,2, 3,4, Inf, Inf), nrow = 2, dimnames = list(c(1,2), c(3,4, "U")))
   expect_equal(length(fullmatch(d)), 5)
-  
+
   # the previous version also returned all 5 entries, not just the matched ones.
   expect_equal(length(fullmatch(as.InfinitySparseMatrix(d))), 5)
 })
@@ -137,7 +137,7 @@ test_that("Reversion Test: Proper labeling of NAs", {
 
   # also, entirely failing problems should be labeled with NAs, not subgroup.NA
   # https://github.com/markmfredrickson/optmatch/issues/22
-  scores <- c(10,1,10,10,10,10,10,10,10, 1,1,1,1,1,10,10,10) 
+  scores <- c(10,1,10,10,10,10,10,10,10, 1,1,1,1,1,10,10,10)
   B <- c(rep(1, 9), rep(2, 8))
   Z <- c(rep(c(0,1), each = 4), 0, rep(c(0,1), each = 4))
 
@@ -147,13 +147,13 @@ test_that("Reversion Test: Proper labeling of NAs", {
   res <- pairmatch(caliper(d, 2))
 
   expect_equal(sum(is.na(res)), 9)
-  
+
   # repeat using an optmatch.dlist object, just in case...
   od <- list(matrix(c(0,0,0,0, Inf,Inf,Inf,Inf, rep(0, 12)), nrow = 4, ncol = 5, dimnames = list(letters[1:4], letters[5:9])),
              matrix(c(0,0,0,0, rep(Inf, 12)), byrow = T, nrow = 4, ncol = 4, dimnames = list(letters[10:13], letters[14:17])))
 
   class(od) <- c("optmatch.dlist", "list")
-  
+
   expect_equal(sum(is.na(pairmatch(od))), 9)
 
   # while we're at it, check that match failed only indicates that 1 level failed
@@ -162,7 +162,7 @@ test_that("Reversion Test: Proper labeling of NAs", {
 
 test_that("Results are in 'data order'", {
   # https://github.com/markmfredrickson/optmatch/issues/14
-  
+
   df <- data.frame(z = rep(c(0,1), 5), x = 1:10, y = rnorm(10))
   df$w <- df$y + rnorm(10)
   rownames(df) <- letters[1:10][sample(1:10)]
@@ -199,7 +199,7 @@ test_that("Results are in 'data order'", {
   tmp <- as.matrix(df)
   rownames(tmp) <- colnames(tmp) <- NULL
   expect_warning(fullmatch(mm, data = tmp), "names")
-  
+
 })
 
 test_that("Complete Inf matrices/ISMs => all NA optmatch object", {
@@ -218,7 +218,7 @@ test_that("Complete Inf matrices/ISMs => all NA optmatch object", {
   res.ism <- fullmatch(ism)
 
   expect_true(all(is.na(res.ism)))
-  
+
 })
 
 test_that("Both mdist and match_on objects accepted", {
@@ -238,12 +238,12 @@ test_that("Both mdist and match_on objects accepted", {
   res.mdist <- fullmatch(tmp)
   res.mon <- fullmatch(match_on(model))
 
-  expect_equivalent(res.mdist, res.mon) 
-  
+  expect_equivalent(res.mdist, res.mon)
+
 })
 
 test_that("full() and pair() are alises to _match functions", {
-  
+
   n <- 14
   Z <- c(rep(0, n/2), rep(1, n/2))
   X1 <- rnorm(n, mean = 5)
@@ -255,4 +255,116 @@ test_that("full() and pair() are alises to _match functions", {
   dists <- match_on(model)
   expect_equivalent(fullmatch(dists), full(dists))
   expect_equivalent(pairmatch(dists), pair(dists))
-}) 
+})
+
+test_that("fullmatch UI cleanup", {
+  n <- 14
+  Z <- c(rep(0, n/2), rep(1, n/2))
+  X1 <- rnorm(n, mean = 5)
+  X2 <- rnorm(n, mean = -2, sd = 2)
+  B <- rep(c(0,1), n/2)
+  test.data <- data.frame(Z, X1, X2, B)
+  rm(Z)
+  rm(X1)
+  rm(X2)
+  rm(B)
+  rm(n)
+
+  m <- match_on(Z~X1 + X2, within=exactMatch(Z~B, data=test.data), data=test.data, caliper=2)
+
+  fm.dist <- fullmatch(m, data=test.data)
+
+  fm.form <- fullmatch(Z~X1 + X2, within=exactMatch(Z~B, data=test.data), data=test.data, caliper=2)
+
+  attr(fm.dist, "call") <- NULL
+  attr(fm.form, "call") <- NULL
+  expect_true(identical(fm.dist, fm.form))
+
+  # with "with()"
+
+  expect_warning(fm.with <- with(data=test.data, fullmatch(Z~X1 + X2, within=exactMatch(Z~B), caliper=2)))
+
+  attr(fm.with, "call") <- NULL
+  expect_true(identical(fm.dist, fm.with))
+
+  # passing a glm
+  ps <- glm(Z~X1+X2, data=test.data, family=binomial)
+
+  fm.ps <- fullmatch(ps, data=test.data, caliper=2)
+
+  fm.glm <- fullmatch(glm(Z~X1+X2, data=test.data, family=binomial), data=test.data, caliper=2)
+  expect_warning(fm.glm2 <- fullmatch(glm(Z~X1+X2, data=test.data, family=binomial), caliper=2))
+  # should be the same, but different group names
+  expect_true(all.equal(attr(fm.glm, "matched.distances"), attr(fm.glm2, "matched.distances")))
+
+  attr(fm.ps, "call") <- NULL
+  attr(fm.glm, "call") <- NULL
+  expect_true(identical(fm.ps, fm.glm))
+
+  # with scores
+
+  ps <- glm(Z~X2, data=test.data, family=binomial)
+
+  m <- match_on(Z ~ X1 + scores(ps), within=exactMatch(Z~B, data=test.data), data=test.data)
+
+  fm.dist <- fullmatch(m, data=test.data)
+
+  fm.form <- fullmatch(Z~ X1 + scores(ps), within=exactMatch(Z~B, data=test.data), data=test.data)
+
+  attr(fm.dist, "call") <- NULL
+  attr(fm.form, "call") <- NULL
+  expect_true(identical(fm.dist, fm.form))
+
+  # passing numeric
+
+  X1 <- test.data$X1
+  Z <- test.data$Z
+
+  names(X1) <- row.names(test.data)
+  names(Z) <- row.names(test.data)
+  fm.vector <- fullmatch(X1,z=Z, data=test.data, caliper=1)
+  expect_warning(fm.vector2 <- fullmatch(X1,z=Z, caliper=1))
+  # should be the same, but different group names
+  expect_true(all.equal(attr(fm.vector, "matched.distances"), attr(fm.vector2, "matched.distances")))
+
+  m <- match_on(X1, z=Z, caliper=1)
+  fm.mi <- fullmatch(m, data=test.data)
+
+  attr(fm.vector, "call") <- NULL
+  attr(fm.mi, "call") <- NULL
+  expect_true(identical(fm.vector, fm.mi))
+
+  # function
+
+  n <- 16
+  Z <- c(rep(0, n/2), rep(1, n/2))
+  X1 <- rep(c(1,2,3,4), each = n/4)
+  B <- rep(c(0,1), n/2)
+  test.data <- data.frame(Z, X1, B)
+  rm(n)
+  rm(Z)
+  rm(X1)
+  rm(B)
+
+  sdiffs <- function(index, data, z) {
+    abs(data[index[,1], "X1"] - data[index[,2], "X1"])
+  }
+
+  result.function <- match_on(sdiffs, z = test.data$Z, data = test.data)
+
+  fm.funcres <- fullmatch(result.function, data=test.data)
+
+  fm.func <- fullmatch(sdiffs, z = test.data$Z, data=test.data)
+  expect_error(fullmatch(sdiffs, z = Z), "A data argument must be given when passing a function")
+
+  attr(fm.funcres, "call") <- NULL
+  attr(fm.func, "call") <- NULL
+  expect_true(identical(fm.funcres, fm.func))
+
+  # passing bad arguments
+
+  expect_error(fullmatch(test.data), "Invalid input, must be a potential argument to match_on")
+  expect_error(fullmatch(TRUE), "Invalid input, must be a potential argument to match_on")
+
+
+})
